class PuppetEnterpriseConsole extends Batman.App
  # Make PuppetEnterpriseConsole available in the global namespace so it can be used
  # as a namespace and bound to in views.
  @global yes

  @set 'isReady', false
  @on 'ready', =>
    @set 'isReady', true
    Timer.start()
    $('abbr').twipsy
      live: true
      html: true
      delayOut: 100
      placement: -> $(this).data('placement') || 'below'

  @observe 'currentRoute', (route) =>
    unless /^(resources|cloning)#/.test(route.options.signature)
      @controllers.get('cloning').cancelCloning()

  @route 'cloning', 'cloning#preview'
  @route 'resources', 'resources#index'
  @route 'resources/:id', 'resources#show', resource: 'resource_types', action: 'show'
  @route 'resources/:type/:title', 'resources#details'
  @route 'orchestration', 'orchestration#index'
  @route 'orchestration/:id', 'orchestration#show', resource: 'agents', action: 'show'
  @route 'puppetd', 'puppetd#show'
  @root 'resources#index'

  @adjustNodeListHeight = ->
    return setTimeout(arguments.callee, 50) unless (nodeSelector = $('#node-selector')).length
    $('.master-node-list').css
      top: nodeSelector.position().top + nodeSelector.outerHeight( true ) # "true" to use margin as well

  @toggleAdvancedSearch: (node) ->
    $(node).toggleClass('expanded')
    $node = $('.search .advanced')
    $node.animate { height: if $node.is(':visible') then 'hide' else 'show' }
      duration: 200
      step: @adjustNodeListHeight
      complete: @adjustNodeListHeight

  @toggleNodeList: (node) ->
    $(node).parents('.expander-parent').find('.node-expander .expandable').toggleClass('expanded')
    $(node).parents('.expander-parent').find('.node-list').animate({ height:'toggle', width: 'toggle' }, 100)

  @expandAgentResults: (node) ->
    parent_table = $(node).parents('.agent-output')[0]
    $(parent_table).find('.agent-summary').toggleClass('inactive')
    $(parent_table).find('.agent-result').toggleClass('inactive')
    $(parent_table).find('.controls').children().toggleClass('inactive')

  @reduceAgentResults: (node) ->
    parent_table = $(node).parents('.agent-output')[0]
    $(parent_table).find('.agent-summary').toggleClass('inactive')
    $(parent_table).find('.agent-result').toggleClass('inactive')
    $(parent_table).find('.controls').children().toggleClass('inactive')

  @toggleAction: (node) ->
    form = $(node).parents('li').find('form')
    @closeActions(form)
    form.slideToggle('fast')
    $(node).parents('li').toggleClass('active')

  @closeActions: (form) ->
    form.parents( 'ul' ).find( 'form' ).not( form ).parent().removeClass( 'active' ).end().slideUp( 'fast' )

  @togglePopover: (target, id) ->
    $popup = $("##{id}")
    $target = $(target)

    return if $popup.is(':visible')

    $popup.attr tabindex: -1
    $popup.blur ->
      $popup.fadeOut 100, ->
        $popup.attr(tabindex: null)

    if $popup.hasClass('left') or $popup.hasClass('right')
      top = $target.position().top + ($target.outerHeight() / 2) - ($popup.outerHeight() / 2)
      left = $target.position().left + (if $popup.hasClass('left') then -($popup.outerWidth() + 4) else $target.outerWidth() + 4)
    else if $popup.hasClass('above') or $popup.hasClass('below')
      left = $target.position().left + ($target.outerWidth() / 2) - ($popup.outerWidth() / 2)
      top = $target.position().top + (if $popup.hasClass('above') then -$popup.outerHeight() else $target.outerHeight())

    left = undefined if left < 0

    $popup.animate({top, left}, 0)

    unless left?
      $popup.find('.arrow').css {
        left: $target.offset().left - $popup.offset().left + $target.width() / 2
      }

    $popup.fadeIn(100)
    $popup.focus()

Batman.DOM.readers.view = Batman.DOM.readers.defineview
Batman.DOM.readers.prefetch = (_, key) ->
  Batman.View.sourceCache.prefetch(Batman.Navigator.normalizePath(Batman.View::prefix, key))

Batman.RestStorage.Filtered = (param, value) ->
  class extends Batman.RestStorage
    @::before 'read', ([_, _, options]) ->
      options[param] = value()
      arguments[0]

    @::before 'readAll', ([_, options]) ->
      options[param] = value()
      arguments[0]

Batman.RestStorage.Filtered.ByNodeSet = Batman.RestStorage.Filtered('nodeset_id', -> Node.validateAndGetNodeSetID())

Batman.Filters.ifUndefined = (value, otherwise) ->
 if value? then value else otherwise

Batman.Filters['if'] = (value, truthy, falsey) ->
  if value then truthy else falsey

Batman.Filters['and'] = (predicate, value) ->
  predicate and value

Batman.Filters['or'] = (predicate, value) ->
  predicate or value

Batman.Filters.json = (value) ->
  JSON.stringify(value)

Batman.Filters.trivial = (value, depth) ->
  if $.isArray(value)
    for result in value
      return false if Batman.Filters.summarize(result[0], depth) != Batman.Filters.prettify(result[0], depth)
    return true

  return Batman.Filters.summarize(value, depth) == Batman.Filters.prettify(value)

Batman.Filters.summarize = (value, depth) ->
  type = Batman.Filters.typeof(value)
  return '<h3><a data-event-click="expandAgentResults">...</a></h3>' if depth == 0 and type != "scalar" and type != "null"

  str = ''
  if type == "null"
    str += '<em class="undefined">&mdash;</em>'
  else if type == "hash"
    str += '<table class="striped">'
    str += "<tr><th>#{k}</th><td>#{ Batman.Filters.summarize(value[k], depth - 1) }</td></tr>" for k in Object.keys(value).sort()
    str += '</table>'
  else if type == "array"
    str += '<ul>'
    str += "<li>#{ Batman.Filters.summarize(k, depth - 1) }</li>" for k in value.sort()
    str += '</ul>'
  else if type == "scalar"
    v = value
    v = "<pre>#{value}</pre>" if typeof(value) == "string" and value.match /\r|\n/
    str += v

  return str

Batman.Filters.typeof = (value) ->
  if !value or (value and !!value.constructor.keys and Object.keys(value).length == 0) or (value and $.isArray(value) and value.length == 0) # null
    return "null"
  else if !!value.constructor.keys # hash
    return "hash"
  else if $.isArray(value) # array
    return "array"
  else
    return "scalar"

Batman.Filters.prettify = (value) ->
  return Batman.Filters.summarize(value, -1)

Batman.Filters.expandArray = (array) ->
  str = ''
  for value in array
    str += "<li>#{ Batman.Filters.prettify(value) }</li>"
  return str

Batman.Filters.expandHash = (hash) ->
  str = ''
  for key, value of hash
    str += """
    <tr>
      <td>#{ key }</td>
      <td>#{ Batman.Filters.prettify(value) }</td>
    </tr>
    """
  return str

Batman.Filters.allEquals = (list, value) ->
  return true unless list?
  container = new Batman.Hash()
  value = JSON.stringify(value)
  list.map (x) -> container.set(JSON.stringify(x) is value)
  return container.keys().indexOf(false) < 0

Batman.Filters.asPercentOf = (fraction, total) ->
  percent = 100 * fraction / total
  percent = 0 if isNaN(percent)
  return percent.toFixed(1) + '%'

Batman.Filters.deepEquals = (a, b) ->
  JSON.stringify(a) is JSON.stringify(b)

Batman.Filters.slice = (list, start, end) ->
  return unless list?
  list = list.toArray() if list.toArray
  list.slice(start, end)

Batman.Filters.curry = (f, params...) ->
  ->
    args = Array::slice.call(arguments)
    event = args.pop()
    args.push(params...)
    args.push(event)
    f.apply this, args

# useful for being able to interact with the thing we log
Batman.Filters.logAndAssignToWindowVariable = (value, variable) ->
  console.log(value)
  window[variable] = value
  value

Batman.Filters.any = (objList, attr) ->
  return false unless objList and objList.length

  for obj in objList
    return true if obj and obj.constructor.get and obj.get(attr)
  return false
