class PuppetEnterpriseConsole.CloningController extends Batman.Controller
  tabs: ['errors', 'failures', 'overwrites', 'creations', 'successes']

  updatePreview: =>
    @set 'cloningState', 'preview'
    @set 'loadingState', 'loading'
    @set 'nodeCount'   , null
    @set 'activeTab'   , null

    @load()

  moveContentTop: (height) -> $('#content-wrapper').css { top: $(this).outerHeight(true) }

  setupClone: (node, type, title, variation) ->
    @unset 'cloningState'
    @unset 'loadingState'
    obj = { type, title }
    obj.parameters = variation.attributes
    obj.nodes = variation.nodes

    alreadyCloning = @get('cloningResource')
    targetNode = $('#cloning-context')
    if JSON.stringify(alreadyCloning) is JSON.stringify(obj)
      return if targetNode.is(':animated')
      targetNode.animate({ top: -5 }, duration: 50)
                .animate({ top: 0 }, duration: 400, easing: 'easeOutBounce')
    else
      if wasHidden = targetNode.is(":hidden")
        targetNode.css({ display: 'block', overflow: 'hidden', height: 0 })

      transfer = $('<div class="ui-effects-transfer"></div>')
        .appendTo(document.body)
        .css({ position: "absolute", padding: '0.5em', height: $(node).height(), width: $(node).width() })

      target =
        top:  [targetNode.offset().top, 'easeOutSine']
        left: [targetNode.offset().left + (targetNode.width() - transfer.width()) / 2, 'easeOutSine']
        opacity: 0
        width: 0

      targetNode.parent().css({ height: targetNode.height() }) if targetNode.height()
      transfer.css
          zIndex:          1000000
          backgroundColor: targetNode.css('backgroundColor')
          top:             $(node).offset().top
          left:            $(node).offset().left
        .animate target,
          easing: 'easeInExpo'
          duration: 300
          complete: =>
            targetNode.animate { height: 'show' },
              duration: 300
              easing: 'easeOutBounce'
              step: @moveContentTop if wasHidden
              complete: ->
                targetNode.parent().css({ height: '' })
                transfer.remove()

      targetNode.fadeOut 100, ->
        targetNode.css({ display: 'none', overflow: '', height: '' })
        PuppetEnterpriseConsole.controllers.cloning.set 'cloningResource', obj

  preview: ->
    return @redirect('/') unless @get('cloningResource')
    PuppetEnterpriseConsole.observe 'currentRoute', @navigateAway
    Node.observeAndFire 'nodeSetID', @updatePreview

  navigateAway: (currentRoute) =>
    unless currentRoute?.options.signature == "cloning#preview"
      Node.forget 'nodeSetID', @updatePreview
      @unset 'cloningState'
      @unset 'loadingState'
      PuppetEnterpriseConsole.forget 'currentRoute', @navigateAway
      PuppetEnterpriseConsole.forget 'currentRoute', @cancelCloning

  clone: ->
    Node.forget 'nodeSetID', @updatePreview
    PuppetEnterpriseConsole.forget 'currentRoute', @navigateAway
    PuppetEnterpriseConsole.observe 'currentRoute', @cancelCloning

    @set 'cloningState', 'cloning'
    @set 'loadingState', 'loading'
    @set 'nodeCount',    Node.get('selected').length
    Resource[@get('cloningResource').type].invalidate()

    @load()

  cancelCloning: =>
    $('#cloning-context').animate { marginTop: 'hide', height: 'hide' },
      duration: 200
      easing: 'easeOutBack'
      step: @moveContentTop
      complete: =>
        @navigateAway()
        @unset 'cloningResource'

  activateFirstTab: =>
    if @cloningState is 'preview'
      result = @get('previewResult')
    else
      result = @get('cloneResult')

    for key in @tabs
      if result['counts'][key]
        @activateTab(null, key)
        break

  activateTab: (_, subject) ->
    result = if @cloningState is 'preview' then @get('previewResult') else @get('cloneResult')
    @set('activeTab', subject)
    view = new Batman.View(source: "cloning/preview/#{subject}", context: result)
    view.on 'ready', ->
      Batman.DOM.replace 'panel', view.get('node')

  load: ->
    @openConnections ||= 0
    @openConnections++
    res = @get('cloningResource')
    $.ajax
      url: if @cloningState is 'preview' then '/lm/clone/preview' else '/lm/clone'
      type: 'POST'
      data:
        type:         res.type
        title:        res.title
        parameters:   res.parameters
        origin_nodes: res.nodes
        nodeset_id:   Node.get('nodeSetID')
      success: (data) =>
        data.sourceData = res
        if @cloningState is 'preview'
          @set 'previewResult', data
        else
          @set 'cloneResult', data
        @activateFirstTab()
      complete: =>
        --@openConnections || @set 'loadingState', 'loaded'
