class PuppetEnterpriseConsole.NodeEncoder
  encode: (nodes) ->
    JSON.stringify(nodes.map((n) -> n.get('name')))
  decode: (nodes) ->
    return nodes unless Node.get('batmanState') == 'loaded'
    Node.get('all').toArray().filter((e) -> nodes.some( (node) -> node == e.get('name')))

class PuppetEnterpriseConsole.Agent extends Batman.Model
  @global true

  @encode 'name', 'description', 'version', 'actions'
  @encode 'nodes', new PuppetEnterpriseConsole.NodeEncoder()
  @primaryKey = 'name'

  @persist Batman.RestStorage
  @url = '/lm/mc_agents'
  url: -> "/lm/mc_agents"

  # Convert array of node names to node objects on initialization of the Node model
  PuppetEnterpriseConsole.on 'ready', =>
    Node.observe 'nodeSetID', =>
      for agent in @get('agents').toArray()
        if agent.get('nodes').length and not agent.get('nodes')[0].constructor.get?
          nodes = Node.get('all').toArray().filter((e) -> agent.get('nodes').some( (node) -> node == e.get('name')))
          agent.set('nodes', nodes)
          @agent(agent.get('name')).set('nodes', nodes)

  @set('loadedAgents', new Batman.Set())
  @classAccessor 'agents'
    get: ->
      loaded = @get('loadedAgents')
      unless loaded.get('state')
        loaded.set('state', 'loading')
        @load (err, agents) ->
          agents = agents.filter((e) -> e.get('actions').length > 0)
          for agent in agents
            action.has_options = !!Object.keys(action.input).length for action in agent.get('actions')
            id = agent.get('name')
            Agent[id] = new Batman.Object() unless Agent[id]?
            Agent[id].set('data', agent)
          loaded.add(agents...)
          loaded.set('state', 'loaded')
      return loaded

  @agent: (id) ->
    unless Agent[id]?
      Agent[id] = new Batman.Object()
    return Agent[id]

  run: (action_name, nodeset_id, params) ->
    results = new Batman.Object()
    # Find a more Batmanish way to do this
    $.ajax '/lm/mc_run'
      async: true
      type: 'POST'
      data: { agent_name: @.get('name'), action_name: action_name, nodeset_id: nodeset_id, input: params }
      success: (data) =>
        # Create a summary list of nodes for each result (used for preview purposes)
        for result in data
          nodes = result[1]
          summary_nodes = []
          i = 0
          while (i < 3 and i < nodes.length)
            summary_nodes.push(nodes[i++])
          result.push(summary_nodes)
          result.push(nodes.length - summary_nodes.length)

        # data looks like [[content, node_list, summary_node_list, node_diff], [content, node_list, summary_node_list, node_diff]]
        results.set('data', data)
    return results
